## ----setup, include=FALSE--------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## --------------------------------------------------------------------------
# Function to create random data (one sample)
d_random <- function(n = 20000, mean = 0, sd = 1, ncol = 20, cofactor = 5) {
  d <- sinh(matrix(rnorm(n, mean, sd), ncol = ncol)) * cofactor
  colnames(d) <- paste0("marker", sprintf("%02d", 1:ncol))
  d
}

# Create random data (without differential signal)
set.seed(123)
d_input <- list(
  sample1 = d_random(), 
  sample2 = d_random(), 
  sample3 = d_random(), 
  sample4 = d_random()
)

# Add DA signal
ix_DA <- 801:900
ix_cols_type <- 1:10
d_input[[3]][ix_DA, ix_cols_type] <- d_random(n = 1000, mean = 2, ncol = 10)
d_input[[4]][ix_DA, ix_cols_type] <- d_random(n = 1000, mean = 2, ncol = 10)

# Add DS signal
ix_DS <- 901:1000
ix_cols_DS <- 19:20
d_input[[1]][ix_DS, ix_cols_type] <- d_random(n = 1000, mean = 3, ncol = 10)
d_input[[2]][ix_DS, ix_cols_type] <- d_random(n = 1000, mean = 3, ncol = 10)
d_input[[3]][ix_DS, c(ix_cols_type, ix_cols_DS)] <- d_random(n = 1200, mean = 3, ncol = 12)
d_input[[4]][ix_DS, c(ix_cols_type, ix_cols_DS)] <- d_random(n = 1200, mean = 3, ncol = 12)

## --------------------------------------------------------------------------
# Experiment information
experiment_info <- data.frame(
  sample_id = factor(paste0("sample", 1:4)), 
  group_id = factor(c("group1", "group1", "group2", "group2")), 
  stringsAsFactors = FALSE
)

experiment_info

# Marker information
marker_info <- data.frame(
  channel_name = paste0("channel", sprintf("%03d", 1:20)), 
  marker_name = paste0("marker", sprintf("%02d", 1:20)), 
  marker_class = factor(c(rep("type", 10), rep("state", 10)), 
                        levels = c("type", "state", "none")), 
  stringsAsFactors = FALSE
)

marker_info

## --------------------------------------------------------------------------
library(diffcyt)

# Create design matrix
design <- createDesignMatrix(experiment_info, cols_design = 2)

# Alternatively: create model formula (required for some methods)
formula <- createFormula(experiment_info, cols_fixed = 2, cols_random = 1)

# Create contrast matrix
contrast <- createContrast(c(0, 1))

## ---- fig.show = 'hide'----------------------------------------------------
# Test for differential abundance (DA) of clusters
out_DA <- diffcyt(d_input, experiment_info, marker_info, 
                  design = design, contrast = contrast, 
                  analysis_type = "DA", method_DA = "diffcyt-DA-edgeR", 
                  seed_clustering = 123, verbose = FALSE)

# Test for differential states (DS) within clusters
out_DS <- diffcyt(d_input, experiment_info, marker_info, 
                  design = design, contrast = contrast, 
                  analysis_type = "DS", method_DS = "diffcyt-DS-limma", 
                  seed_clustering = 123, plot = FALSE, verbose = FALSE)

# Display results for top DA clusters
topClusters(out_DA$res)

# Number of significant detected DA clusters at 10% FDR
threshold <- 0.1
res_DA_all <- topClusters(out_DA$res, all = TRUE)
table(res_DA_all$p_adj <= threshold)

# Display results for top DS cluster-marker combinations
topClusters(out_DS$res)

# Number of significant detected DS cluster-marker combinations at 10% FDR
threshold <- 0.1
res_DS_all <- topClusters(out_DS$res, all = TRUE)
table(res_DS_all$p_adj <= threshold)

## --------------------------------------------------------------------------
# Prepare data
d_se <- prepareData(d_input, experiment_info, marker_info)

## --------------------------------------------------------------------------
# Transform data
d_se <- transformData(d_se)

## --------------------------------------------------------------------------
# Generate clusters
d_se <- generateClusters(d_se, seed_clustering = 123)

## --------------------------------------------------------------------------
# Calculate counts
d_counts <- calcCounts(d_se)

# Calculate medians
d_medians <- calcMedians(d_se)

## --------------------------------------------------------------------------
# Create design matrix
design <- createDesignMatrix(experiment_info, cols_design = 2)

# Alternatively: create model formula (required for some methods)
formula <- createFormula(experiment_info, cols_fixed = 2, cols_random = 1)

# Create contrast matrix
contrast <- createContrast(c(0, 1))

## --------------------------------------------------------------------------
# Test for differential abundance (DA) of clusters
res_DA <- testDA_edgeR(d_counts, design, contrast)

# Display results for top DA clusters
topClusters(res_DA)

# Number of significant detected DA clusters at 10% FDR
threshold <- 0.1
res_DA_all <- topClusters(res_DA, all = TRUE)
table(res_DA_all$p_adj <= threshold)

## ---- fig.show = 'hide'----------------------------------------------------
# Test for differential states (DS) within clusters
res_DS <- testDS_limma(d_counts, d_medians, design, contrast, plot = FALSE)

# Display results for top DS cluster-marker combinations
topClusters(res_DS)

# Number of significant detected DS cluster-marker combinations at 10% FDR
threshold <- 0.1
res_DS_all <- topClusters(res_DS, all = TRUE)
table(res_DS_all$p_adj <= threshold)

## --------------------------------------------------------------------------
suppressPackageStartupMessages(library(SummarizedExperiment))
suppressPackageStartupMessages(library(ggplot2))
suppressPackageStartupMessages(library(reshape2))
suppressPackageStartupMessages(library(ggridges))

# data frame for 'ggplot'
d_plot <- cbind(as.data.frame(assay(d_se)), as.data.frame(rowData(d_se)))
d_plot <- melt(d_plot, id.vars = c("sample_id", "group_id", "cluster_id"), 
               variable.name = "marker", value.name = "expression")

# ridgeline plot of marker expression profiles
ggplot(d_plot, aes(x = expression, y = marker, fill = group_id)) + 
  geom_density_ridges(alpha = 0.5) + 
  scale_fill_manual(values = c("orangered", "royalblue")) + 
  theme_bw()

## --------------------------------------------------------------------------
# Plot heatmap for DA tests
plotHeatmap(out_DA, analysis_type = "DA")

# Plot heatmap for DS tests
plotHeatmap(out_DS, analysis_type = "DS")

